@@ -23,11 +23,6 @@ import shutil
 import stat
 import grp
 import pwd
-try:
-    import selinux
-    HAVE_SELINUX=True
-except ImportError:
-    HAVE_SELINUX=False
 
 DOCUMENTATION = '''
 ---
@@ -164,8 +159,8 @@ def main():
             original_basename = dict(required=False), # Internal use only, for recursive ops
             recurse  = dict(default=False, type='bool'),
             force = dict(required=False, default=False, type='bool'),
-            diff_peek = dict(default=None),
-            validate = dict(required=False, default=None),
+            diff_peek = dict(default=None), # Internal use only, for internal checks in the action plugins
+            validate = dict(required=False, default=None), # Internal use only, for template and copy
             src = dict(required=False, default=None),
         ),
         add_file_common_args=True,
@@ -197,6 +192,7 @@ def main():
 
     prev_state = get_state(path)
 
+
     # state should default to file, but since that creates many conflicts,
     # default to 'current' when it exists.
     if state is None:
@@ -233,10 +229,23 @@ def main():
         module.fail_json(path=path, msg="recurse option requires state to be 'directory'")
 
     file_args = module.load_file_common_arguments(params)
+
     changed = False
+    diff = {'before':
+                {'path': path}
+            ,
+            'after':
+                {'path': path}
+            }
+
+    state_change = False
+    if prev_state != state:
+        diff['before']['state'] = prev_state
+        diff['after']['state'] = state
+        state_change = True
 
     if state == 'absent':
-        if state != prev_state:
+        if state_change:
             if not module.check_mode:
                 if prev_state == 'directory':
                     try:
@@ -248,13 +257,13 @@ def main():
                         os.unlink(path)
                     except Exception, e:
                         module.fail_json(path=path, msg="unlinking failed: %s " % str(e))
-            module.exit_json(path=path, changed=True)
+            module.exit_json(path=path, changed=True, diff=diff)
         else:
             module.exit_json(path=path, changed=False)
 
     elif state == 'file':
 
-        if state != prev_state:
+        if state_change:
             if follow and prev_state == 'link':
                 # follow symlink and operate on original
                 path = os.path.realpath(path)
@@ -265,8 +274,8 @@ def main():
             # file is not absent and any other state is a conflict
             module.fail_json(path=path, msg='file (%s) is %s, cannot continue' % (path, prev_state))
 
-        changed = module.set_fs_attributes_if_different(file_args, changed)
-        module.exit_json(path=path, changed=changed)
+        changed = module.set_fs_attributes_if_different(file_args, changed, diff)
+        module.exit_json(path=path, changed=changed, diff=diff)
 
     elif state == 'directory':
         if follow and prev_state == 'link':
@@ -275,7 +284,7 @@ def main():
 
         if prev_state == 'absent':
             if module.check_mode:
-                module.exit_json(changed=True)
+                module.exit_json(changed=True, diff=diff)
             changed = True
             curpath = ''
 
@@ -295,11 +304,11 @@ def main():
                         except OSError, ex:
                             # Possibly something else created the dir since the os.path.exists
                             # check above. As long as it's a dir, we don't need to error out.
-                            if not (ex.errno == errno.EEXIST and os.isdir(curpath)):
+                            if not (ex.errno == errno.EEXIST and os.path.isdir(curpath)):
                                 raise
                         tmp_file_args = file_args.copy()
                         tmp_file_args['path']=curpath
-                        changed = module.set_fs_attributes_if_different(tmp_file_args, changed)
+                        changed = module.set_fs_attributes_if_different(tmp_file_args, changed, diff)
             except Exception, e:
                 module.fail_json(path=path, msg='There was an issue creating %s as requested: %s' % (curpath, str(e)))
 
@@ -307,12 +316,12 @@ def main():
         elif prev_state != 'directory':
             module.fail_json(path=path, msg='%s already exists as a %s' % (path, prev_state))
 
-        changed = module.set_fs_attributes_if_different(file_args, changed)
+        changed = module.set_fs_attributes_if_different(file_args, changed, diff)
 
         if recurse:
             changed |= recursive_set_attributes(module, file_args['path'], follow, file_args)
 
-        module.exit_json(path=path, changed=changed)
+        module.exit_json(path=path, changed=changed, diff=diff)
 
     elif state in ['link','hard']:
 
@@ -381,10 +390,10 @@ def main():
                     module.fail_json(path=path, msg='Error while linking: %s' % str(e))
 
         if module.check_mode and not os.path.exists(path):
-            module.exit_json(dest=path, src=src, changed=changed)
+            module.exit_json(dest=path, src=src, changed=changed, diff=diff)
 
-        changed = module.set_fs_attributes_if_different(file_args, changed)
-        module.exit_json(dest=path, src=src, changed=changed)
+        changed = module.set_fs_attributes_if_different(file_args, changed, diff)
+        module.exit_json(dest=path, src=src, changed=changed, diff=diff)
 
     elif state == 'touch':
         if not module.check_mode:
@@ -402,7 +411,7 @@ def main():
             else:
                 module.fail_json(msg='Cannot touch other than files, directories, and hardlinks (%s is %s)' % (path, prev_state))
             try:
-                module.set_fs_attributes_if_different(file_args, True)
+                module.set_fs_attributes_if_different(file_args, True, diff)
             except SystemExit, e:
                 if e.code:
                     # We take this to mean that fail_json() was called from
@@ -412,7 +421,7 @@ def main():
                         os.remove(path)
                 raise e
 
-        module.exit_json(dest=path, changed=True)
+        module.exit_json(dest=path, changed=True, diff=diff)
 
     module.fail_json(path=path, msg='unexpected position reached')
 
