@@ -38,6 +38,12 @@ from ansible.utils.boolean import boolean
 from ansible.utils.vars import combine_vars, isidentifier
 from ansible.utils.unicode import to_unicode
 
+try:
+    from __main__ import display
+except ImportError:
+    from ansible.utils.display import Display
+    display = Display()
+
 BASE_ATTRIBUTES = {}
 
 
@@ -153,7 +159,7 @@ class Base:
             setattr(Base, name, property(getter, setter, deleter))
 
             # Place the value into the instance so that the property can
-            # process and hold that value/
+            # process and hold that value.
             setattr(self, name, value.default)
 
     def preprocess_data(self, ds):
@@ -304,6 +310,8 @@ class Base:
                 method = getattr(self, '_post_validate_%s' % name, None)
                 if method:
                     value = method(attribute, getattr(self, name), templar)
+                elif attribute.isa == 'class':
+                    value = getattr(self, name)
                 else:
                     # if the attribute contains a variable, template it now
                     value = templar.template(getattr(self, name))
@@ -330,11 +338,15 @@ class Base:
                         if isinstance(value, string_types) and '%' in value:
                             value = value.replace('%', '')
                         value = float(value)
-                    elif attribute.isa == 'list':
+                    elif attribute.isa in ('list', 'barelist'):
                         if value is None:
                             value = []
                         elif not isinstance(value, list):
-                            if isinstance(value, string_types):
+                            if isinstance(value, string_types) and attribute.isa == 'barelist':
+                                display.deprecated(
+                                    "Using comma separated values for a list has been deprecated. " \
+                                    "You should instead use the correct YAML syntax for lists. " \
+                                )
                                 value = value.split(',')
                             else:
                                 value = [ value ]
@@ -363,6 +375,10 @@ class Base:
                             value = dict()
                         elif not isinstance(value, dict):
                             raise TypeError("%s is not a dictionary" % value)
+                    elif attribute.isa == 'class':
+                        if not isinstance(value, attribute.class_type):
+                            raise TypeError("%s is not a valid %s (got a %s instead)" % (name, attribute.class_type, type(value)))
+                        value.post_validate(templar=templar)
 
                 # and assign the massaged value back to the attribute field
                 setattr(self, name, value)
